# -*- coding: utf-8 -*-

import ephem
from . import nutation
import pytz
from math import pi, floor
from datetime import timedelta

ABERR_CONST = (20.49552 / 3600. / 180. * pi)

twopi = pi * 2.
twelfth_pi = pi / 12.
twenty_fourth_pi = pi / 24.

degree = pi / 180.
arcminute = degree / 60.
arcsecond = arcminute / 60.
half_arcsecond = arcsecond / 2.
tiny = arcsecond / 360.


_sun = ephem.Sun()    # for computing equinoxes


def get_ap_hlon(mj, nutation_dpsi=None):
    _sun.compute(mj)
    if nutation_dpsi is None: nutation_dpsi = nutation.nutation(mj)[1]
    ap_hlon = _sun.hlon + nutation_dpsi - ABERR_CONST + pi
    if ap_hlon < 0.0:
        ap_hlon += twopi
    elif ap_hlon > twopi:
        ap_hlon -= twopi
    return ephem.degrees(ap_hlon)

def converge(d, deg):

    def get_diff(d, deg, nutation_dpsi=None):
        diff = float(deg) * degree - get_ap_hlon(d, nutation_dpsi)
        if diff > pi:
            diff -= twopi
        elif diff < -pi:
            diff += twopi
        return diff

    # converging iterations using the get_diff() function, not fixing nutation
    for i in range(100):
        diff = get_diff(d, deg)
        if abs(diff) < ephem.degrees('0:00:01'):
            break
        d = d + 365.25 * diff / twopi  # update d using the difference

    # apply the bisection method, fixing nutation
    nutation_dpsi = nutation.nutation(d)[1]
    d0, d1 = d-ephem.degrees('0:05:00'), d+ephem.degrees('0:05:00')
    f0, f1 = get_diff(d0, deg, nutation_dpsi), get_diff(d1, deg, nutation_dpsi)
    if f0 * f1 > 0.:
        # TODO: use Exception
        print("warning: f0=%f, f1=%f" % (f0, f1))
        sys.exit(0)

    for i in range(20):  # limits the iteration number to 20.
        
        dn = (d0 + d1) / 2.
        fn = get_diff(dn, deg, nutation_dpsi)

        if fn * f0 > 0.:  # the midpoint has the same sign as the left side -> select the right side
            d0 = dn
            f0 = get_diff(d0, deg, nutation_dpsi)
        elif fn * f1 > 0.:  # the midpoint has the same sign as the right side -> select the left side
            d1 = dn
            f1 = get_diff(d1, deg, nutation_dpsi)
        elif fn == 0:
            return ephem.date(dn)
        else:
            # TODO: use Exception
            print("warning")
            sys.exit(0)

    return ephem.Date((d0*abs(f1)+d1*abs(f0))/(abs(f0) + abs(f1)))


def solar_term_finder(mj, n, reverse=False):
    
    offset = n * twelfth_pi
    d0 = ephem.Date(mj)
    motion = -twopi if reverse else twopi

    angle_to_cover = motion - (get_ap_hlon(d0) - offset) % motion
    if abs(angle_to_cover) < tiny:
        angle_to_cover = motion
    d = d0 + 365.25 * angle_to_cover / twopi
    return converge(d, n * 15)


def solar_term_finder_deg(mj, deg, reverse=False):

    offset = deg * degree
    d0 = ephem.Date(mj)
    motion = -twopi if reverse else twopi

    angle_to_cover = motion - (get_ap_hlon(d0) - offset) % motion
    if abs(angle_to_cover) < tiny:
        angle_to_cover = motion
    d = d0 + 365.25 * angle_to_cover / twopi
    return converge(d, deg)


def solar_term_finder_adjacent(mj, divisor=30.0, remainder=15.0, reverse=False):

    d0 = ephem.Date(mj)
    if reverse:
        deg = (floor((get_ap_hlon(d0) / degree - remainder) / divisor)) * divisor + remainder
    else:
        deg = (floor((get_ap_hlon(d0) / degree - remainder) / divisor) + 1) * divisor + remainder
    
    angle_to_cover = deg * degree - get_ap_hlon(d0)

    d = d0 + 365.25 * angle_to_cover / twopi
    return (int((deg % 360.0) / 15.0), deg % 360.0, converge(d, deg))


def annual_solar_terms(year, boundary_previous=False, boundary_following=False):

    ref = ephem.previous_winter_solstice(str(year)) + 0.01

    result = []
    for j in range(24):
        i = (j - 5) % 24
        d = pytz.utc.localize(solar_term_finder(ref, i).datetime())
        result.append((i, d))

    if boundary_previous:
        result.insert(0, (18, pytz.utc.localize(solar_term_finder(ref, 18, reverse=True).datetime())))
        result.insert(0, (17, pytz.utc.localize(solar_term_finder(ref, 17, reverse=True).datetime())))
    if boundary_following:
        ref2 = result[-1][1]
        result.append((19, pytz.utc.localize(solar_term_finder(ref2, 19).datetime())))
        result.append((20, pytz.utc.localize(solar_term_finder(ref2, 20).datetime())))

    return result


def annual_jp_doyo_days(year):
    
    ref = ephem.previous_winter_solstice(str(year)) + 0.01

    result = []
    for j in range(4):
        deg_start = (j * 90 + 27 - 90) % 360
        deg_end = (j * 90 + 45 - 90) % 360
        result.append((j+1, 
                       pytz.utc.localize(solar_term_finder_deg(ref, deg_start).datetime()),
                       pytz.utc.localize(solar_term_finder_deg(ref, deg_end).datetime())))

    return result


def annual_jp_higan_days(year):

    ref = ephem.previous_winter_solstice(str(year)) + 0.01

    result = []
    for j in range(2):
        i = j * 12
        d = pytz.utc.localize(solar_term_finder(ref, i).datetime())
        result.append((j+11, 
                       d - timedelta(days=3),
                       d + timedelta(days=3)))

    return result

def annual_jp_seasonal_days(year):

    ref = ephem.previous_winter_solstice(str(year)) + 0.01

    result = []
    
    # Setsubun (節分, the day before the start of spring)
    result.append((101, pytz.utc.localize(solar_term_finder(ref, 21).datetime()) - timedelta(days=1)))

    # Hachiju-hachi-ya (八十八夜, the 88th night after the start of spring)
    result.append((102, pytz.utc.localize(solar_term_finder(ref, 21).datetime()) + timedelta(days=87)))

    # Nihyaku-toka (二百十日, the 210th day after the start of spring)
    result.append((103, pytz.utc.localize(solar_term_finder(ref, 21).datetime() + timedelta(days=209))))
    
    # Nyubai (入梅, deg=80)
    result.append((111, pytz.utc.localize(solar_term_finder_deg(ref, 80).datetime())))

    # Hangesho (半夏生, deg=100)
    result.append((112, pytz.utc.localize(solar_term_finder_deg(ref, 100).datetime())))

    return result



# -*- coding: utf-8 -*-

from . import cycle
from . import solar_terms
from .lang import Lang, str_solar_terms, str_cycle, str_jp_seasonal_days

import pytz
from datetime import datetime, timedelta


class EACal:

    def __init__(self, lang=Lang.EN, tz=pytz.utc,
                 zh_t=False, zh_s=False,
                 ja=False, ko=False,
                 vi=False):
        self.lang = lang
        self.tz = tz
        if zh_t:
            self.lang = Lang.ZH_HANT
            self.tz = pytz.timezone('Asia/Hong_Kong')
        elif zh_s:
            self.lang = Lang.ZH_HANS
            self.tz = pytz.timezone('Asia/Shanghai')
        elif ja:
            self.lang = Lang.JA
            self.tz = pytz.timezone('Asia/Tokyo')
        elif ko:
            self.lang = Lang.KO
            self.tz = pytz.timezone('Asia/Seoul')
        elif vi:
            self.lang = Lang.VI
            self.tz = pytz.timezone('Asia/Ho_Chi_Minh')

    def get_cycle_year(self, year):
        return str_cycle(cycle.cycle_year(year), self.lang)

    def get_cycle_month(self, year, month):
        return str_cycle(cycle.cycle_month(year, month), self.lang)

    def get_cycle_day(self, date):
        return str_cycle(cycle.cycle_day(date), self.lang)

    def get_cycle_ymd(self, dt, id=False):

        # if "dt" is a tuple, it is interpreted as (year, month, day)
        if isinstance(dt, tuple):
            dt = datetime(dt[0], dt[1], dt[2])
        
        # if "dt" is a naive datetime, it is interpreted using 
        # the specified timezone.
        if dt.tzinfo is None:
            dt = self.tz.localize(dt)

        (y_id, m_id, d_id) = cycle.cycle_ymd(dt)

        if id:
            return (y_id, m_id, d_id)
        else:
            return (str_cycle(y_id, self.lang), 
                    str_cycle(m_id, self.lang), 
                    str_cycle(d_id, self.lang))

    def get_annual_solar_terms(self, year,
                               boundary_previous=False, 
                               boundary_following=False):

        st_list = solar_terms.annual_solar_terms(year, 
                                                 boundary_previous,
                                                 boundary_following)
        result = []
        for st in st_list:
            (st_id, st_dt_utc) = st
            st_str = str_solar_terms(st_id, self.lang)
            st_dt_local = st_dt_utc.astimezone(self.tz)
            st_new = (st_str, st_id, st_dt_local)
            result.append(st_new)
        return result

    def get_annual_jp_seasonal_days(self, year):
        
        doyo_days = solar_terms.annual_jp_doyo_days(year)
        higan_days = solar_terms.annual_jp_higan_days(year)
        other_days = solar_terms.annual_jp_seasonal_days(year)

        result = []

        for (d_id, d_start, d_end) in doyo_days:
            d_str = str_jp_seasonal_days(d_id, self.lang)
            d_local_start = d_start.astimezone(self.tz)
            d_local_end   = d_end.astimezone(self.tz)
            d_new = (d_str, d_id, d_local_start, d_local_end)
            result.append(d_new)

        for (d_id, d_start, d_end) in higan_days:
            d_str = str_jp_seasonal_days(d_id, self.lang)
            d_local_start = d_start.astimezone(self.tz).replace(hour=0, minute=0, second=0, microsecond=0)
            d_local_end   = d_end.astimezone(self.tz).replace(hour=0, minute=0, second=0, microsecond=0) + timedelta(days=1)
            d_new = (d_str, d_id, d_local_start, d_local_end)
            result.append(d_new)

        for (d_id, d_dt) in other_days:
            d_str = str_jp_seasonal_days(d_id, self.lang)
            if d_id in [101, 102, 103]:
                d_local_dt = d_dt.astimezone(self.tz).replace(hour=0, minute=0, second=0, microsecond=0)
            else:
                d_local_dt = d_dt.astimezone(self.tz)
            d_new = (d_str, d_id, d_local_dt)
            result.append(d_new)

        return result
            
            



# -*- coding: utf-8 -*-

from . import solar_terms

import ephem
import pytz
import jdcal
from datetime import timedelta

def cycle_year(year):
    return (year - 4) % 60

def cycle_month(year, month):
    return (year * 12 + month + 12) % 60

def cycle_day(date):
    return (int(jdcal.gcal2jd(date.year, date.month, date.day)[1]) + 50) % 60

def cycle_ymd(date):
    date_start = date.replace(hour=0, minute=0, second=0, microsecond=0) + timedelta(days=1)
    d = ephem.Date(date_start.astimezone(pytz.utc))
    (y_n, y_deg, y_d0) = solar_terms.solar_term_finder_adjacent(d, divisor=360.0,
                                                                remainder=315.0, reverse=True)
    (m_n, m_deg, m_d0) = solar_terms.solar_term_finder_adjacent(d, reverse=True)
    date_ym = m_d0.datetime() + timedelta(days=15)
    
    return (cycle_year(y_d0.datetime().year), cycle_month(date_ym.year, date_ym.month), cycle_day(date))




# -*- coding: utf-8 -*-

class Lang:

    EN       =  10

    ZH_HANT  = 210
    ZH_HANS  = 220
    ZH_P     = 290
    
    JA       = 310
    JA_P     = 390
    
    KO       = 410
    KO_HANJA = 420
    KO_P     = 490

    VI       = 510

SolarTerms = {}

SolarTerms[Lang.EN] = 'vernal equinox|clear and bright|grain rain|start of summer|grain full|grain in ear|summer solstice|minor heat|major heat|start of autumn|limit of heat|white dew|autumnal equinox|cold dew|frost descent|start of winter|minor snow|major snow|winter solstice|minor cold|major cold|start of spring|rain water|awakening of insects'.split('|')

SolarTerms[Lang.ZH_HANT] = u'春分|清明|穀雨|立夏|小滿|芒種|夏至|小暑|大暑|立秋|處暑|白露|秋分|寒露|霜降|立冬|小雪|大雪|冬至|小寒|大寒|立春|雨水|驚蟄'.split('|')
SolarTerms[Lang.ZH_HANS] = u'春分|清明|谷雨|立夏|小满|芒种|夏至|小暑|大暑|立秋|处暑|白露|秋分|寒露|霜降|立冬|小雪|大雪|冬至|小寒|大寒|立春|雨水|惊蛰'.split('|')
SolarTerms[Lang.ZH_P] = u'chūnfēn|qīngmíng|gǔyǔ|lìxià|xiǎomǎn|mángzhòng|xiàzhì|xiǎoshǔ|dàshǔ|lìqiū|chǔshǔ|báilù|qiūfēn|hánlù|shuāngjiàng|lìdōng|xiǎoxuě|dàxuě|dōngzhì|xiǎohán|dàhán|lìchūn|yǔshuǐ|jīngzhé'.split('|')

SolarTerms[Lang.JA] = u'春分|清明|穀雨|立夏|小満|芒種|夏至|小暑|大暑|立秋|処暑|白露|秋分|寒露|霜降|立冬|小雪|大雪|冬至|小寒|大寒|立春|雨水|啓蟄'.split('|')
SolarTerms[Lang.JA_P] = u'risshun|usui|keichitsu|shunbun|seimei|kokuu|rikka|shōman|bōshu|geshi|shōsho|taisho|risshū|shosho|hakuro|shūbun|kanro|sōkō|rittō|shōsetsu|taisetsu|tōji|shōkan|daikan'.split('|')

SolarTerms[Lang.KO] = u'춘분|청명|곡우|입하|소만|망종|하지|소서|대서|입추|처서|백로|추분|한로|상강|입동|소설|대설|동지|소한|대한|입춘|우수|경칩'.split('|')
SolarTerms[Lang.KO_HANJA] = u'春分|清明|穀雨|立夏|小滿|芒種|夏至|小暑|大暑|立秋|處暑|白露|秋分|寒露|霜降|立冬|小雪|大雪|冬至|小寒|大寒|立春|雨水|驚蟄'.split('|')
SolarTerms[Lang.KO_P] = u'chunbun|cheongmyeong|gogu|ipha|soman|mangjong|haji|soseo|daeseo|ipchu|cheoseo|baengno|chubun|hallo|sanggang|ipdong|soseol|daeseol|dongji|sohan|daehan|ipchun|usu|gyeongchip'.split('|')

SolarTerms[Lang.VI] = u'Xuân phân|Thanh minh|Cốc vũ|Lập hạ|Tiểu mãn|Mang chủng|Hạ chí|Tiểu thử|Đại thử|Lập thu|Xử thử|Bạch lộ|Thu phân|Hàn lộ|Sương giáng|Lập đông|Tiểu tuyết|Đại tuyết|Đông chí|Tiểu hàn|Đại hàn|Lập xuân|Vũ thủy|Kinh trập'.split('|')


Stems = {}

Stems[Lang.EN] = 'wood-yang|wood-yin|fire-yang|fire-yin|earth-yang|earth-yin|metal-yang|metal-yin|water-yang|water-yin'.split('|')

Stems[Lang.ZH_HANT] = u'甲|乙|丙|丁|戊|己|庚|辛|壬|癸'.split('|')
Stems[Lang.ZH_HANS] = u'甲|乙|丙|丁|戊|己|庚|辛|壬|癸'.split('|')
Stems[Lang.ZH_P] = u'jiǎ|yǐ|bǐng|dīng|wù|jǐ|gēng|xīn|rén|guǐ'.split('|')

Stems[Lang.JA] = u'甲|乙|丙|丁|戊|己|庚|辛|壬|癸'.split('|')
Stems[Lang.JA+1] = u'きのえ|きのと|ひのえ|ひのと|つちのえ|つちのと|かのえ|かのと|みずのえ|みずのと'.split('|')
Stems[Lang.JA_P] = u'kō|otsu|hei|tei|bo|ki|kō|shin|jin|ki'.split('|')
Stems[Lang.JA_P+1] = u'kinoe|kinoto|hinoe|hinoto|tsuchinoe|tsuchinoto|kanoe|kanoto|mizunoe|mizunoto'.split('|')

Stems[Lang.KO] = u'갑|을|병|정|무|기|경|신|임|계'.split('|')
Stems[Lang.KO_HANJA] = u'甲|乙|丙|丁|戊|己|庚|辛|壬|癸'.split('|')
Stems[Lang.KO_P] = u'gap|eul|byeong|jeong|mu|gi|gyeong|sin|im|gye'.split('|')

Stems[Lang.VI] = u'giáp|ất|bính|đinh|mậu|kỷ|canh|tân|nhâm|quý'.split('|')


Branches = {}

Branches[Lang.EN] = 'rax|ox|tiger|rabbit|dragon|snake|horse|goat|monkey|rooster|dog|pig'.split('|')

Branches[Lang.ZH_HANT] = u'子|丑|寅|卯|辰|巳|午|未|申|酉|戌|亥'.split('|')
Branches[Lang.ZH_HANS] = u'子|丑|寅|卯|辰|巳|午|未|申|酉|戌|亥'.split('|')
Branches[Lang.ZH_P] = u'zǐ|chǒu|yín|mǎo|chén|sì|wǔ|wèi|shēn|yǒu|xū|hài'.split('|')

Branches[Lang.JA] = u'子|丑|寅|卯|辰|巳|午|未|申|酉|戌|亥'.split('|')
Branches[Lang.JA+1] = u'ね|うし|とら|う|たつ|み|うま|ひつじ|さる|とり|いぬ|い'.split('|')
Branches[Lang.JA+2] = u'鼠|牛|虎|兎|龍|蛇|馬|羊|猿|鶏|犬|猪'.split('|')
Branches[Lang.JA_P] = u'shi|chū|in|bō|shin|shi|go|bi|shin|yū|jutsu|gai'.split('|')
Branches[Lang.JA_P+1] = u'ne|ushi|tora|u|tatsu|mi|uma|hitsuji|saru|tori|inu|i'.split('|')

Branches[Lang.KO] = u'자|축|인|묘|진|사|오|미|신|유|술|해'.split('|')
Branches[Lang.KO_HANJA] = u'子|丑|寅|卯|辰|巳|午|未|申|酉|戌|亥'.split('|')
Branches[Lang.KO_P] = u'ja|chuk|in|myo|jin|sa|o|mi|sin|yu|sul|hae'.split('|')

Branches[Lang.VI] = u'tý|sửu|dần|mão|thìn|tỵ|ngọ|mùi|thân|dậu|tuất|hợi'.split('|')


Cycle = {}

Cycle[Lang.EN] = ['%s %s' % (Stems[Lang.EN][i % 10],
                             Branches[Lang.EN][i % 12])
                  for i in range(60)]

Cycle[Lang.ZH_HANT] = ['%s%s' % (Stems[Lang.ZH_HANT][i % 10],
                                 Branches[Lang.ZH_HANT][i % 12])
                       for i in range(60)]
Cycle[Lang.ZH_HANS] = ['%s%s' % (Stems[Lang.ZH_HANS][i % 10],
                                 Branches[Lang.ZH_HANS][i % 12])
                       for i in range(60)]
Cycle[Lang.ZH_P] = ['%s %s' % (Stems[Lang.ZH_P][i % 10],
                               Branches[Lang.ZH_P][i % 12])
                    for i in range(60)]

Cycle[Lang.JA] = ['%s%s' % (Stems[Lang.JA][i % 10],
                            Branches[Lang.JA][i % 12])
                  for i in range(60)]
Cycle[Lang.JA+1] = ['%s%s' % (Stems[Lang.JA+1][i % 10],
                              Branches[Lang.JA+1][i % 12])
                    for i in range(60)]
Cycle[Lang.JA_P] = ['%s %s' % (Stems[Lang.JA_P][i % 10],
                               Branches[Lang.JA_P][i % 12])
                    for i in range(60)]
Cycle[Lang.JA_P+1] = ['%s %s' % (Stems[Lang.JA_P+1][i % 10],
                                 Branches[Lang.JA_P+1][i % 12])
                      for i in range(60)]

Cycle[Lang.KO] = ['%s%s' % (Stems[Lang.KO][i % 10],
                            Branches[Lang.KO][i % 12])
                  for i in range(60)]
Cycle[Lang.KO_HANJA] = ['%s%s' % (Stems[Lang.KO_HANJA][i % 10],
                                  Branches[Lang.KO_HANJA][i % 12])
                        for i in range(60)]
Cycle[Lang.KO_P] = ['%s %s' % (Stems[Lang.KO_P][i % 10],
                               Branches[Lang.KO_P][i % 12])
                    for i in range(60)]

Cycle[Lang.VI] = ['%s %s' % (Stems[Lang.VI][i % 10],
                             Branches[Lang.VI][i % 12])
                  for i in range(60)]


JPSeasonalDays = {}

JPSeasonalDays[Lang.EN] = {1: u'doyō:winter',
                           2: u'doyō:spring',
                           3: u'doyō:summer',
                           4: u'doyō:autumn',
                           11: u'higan:spring',
                           12: u'higan:autumn',
                           101: u'setsubun:the day before the start of spring',
                           102: u'hachijū-hachi-ya:the 88th night after the start of spring',
                           103: u'nihyaku-tōka:the 210th day after the start of spring',
                           111: u'nyūbai:the beginning of rainy season',
                           112: u'hangeshō:the end of field work'}

JPSeasonalDays[Lang.JA] = {1: u'土用:冬',
                           2: u'土用:春',
                           3: u'土用:夏',
                           4: u'土用:秋',
                           11: u'彼岸:春',
                           12: u'彼岸:秋',
                           101: u'節分',
                           102: u'八十八夜',
                           103: u'二百十日',
                           111: u'入梅',
                           112: u'半夏生'}
                    

def str_solar_terms(solar_term_id, lang):
    return SolarTerms[lang][solar_term_id]

def str_stems(stem_id, lang, ja_kun_yomi=False):
    if lang == Lang.JA and ja_kun_yomi:
        _lang = Lang.JA + 1
    elif lang == Lang.JA_P and ja_kun_yomi:
        _lang = Lang.JA_P + 1
    else:
        _lang = lang
    return Stems[_lang][stem_id]

def str_branches(branch_id, lang, ja_kun_yomi=False):
    if lang == Lang.JA and ja_kun_yomi:
        _lang = Lang.JA + 1
    elif lang == Lang.JA_P and ja_kun_yomi:
        _lang = Lang.JA_P + 1
    else:
        _lang = lang
    return Branches[_lang][branch_id]
    
def str_cycle(cycle_id, lang, ja_kun_yomi=False):
    if lang == Lang.JA and ja_kun_yomi:
        _lang = Lang.JA + 1
    elif lang == Lang.JA_P and ja_kun_yomi:
        _lang = Lang.JA_P + 1
    else:
        _lang = lang
    return Cycle[_lang][cycle_id]
    
def str_jp_seasonal_days(day_id, lang):
    if lang == Lang.JA:
        _lang = Lang.JA
    else:
        _lang = Lang.EN
    return JPSeasonalDays[_lang][day_id]
    



# -*- coding: utf-8 -*-

from distutils.core import setup

long_description = '''\
EastAsiaCalendars README
========================

EastAsiaCalendars provides an ``eacal`` Python module for accessing East
Asia calendar systems, originated in China, and spread to Korea,
Vietnam, and Japan. This module includes the following.

-  `solar terms <http://en.wikipedia.org/wiki/Solar_term>`__ (節氣,
   节气, 節気, 절기, tiết khí)
-  `sexagenary cycle <http://en.wikipedia.org/wiki/Sexagenary_cycle>`__
   (六十花甲, 干支, 간지, Gānzhī)
-  `zassetsu <http://ja.wikipedia.org/wiki/%E9%9B%91%E7%AF%80>`__ (雑節,
   Seasonal days in the Japanese calendar)

Solar terms are calculated based on the planetary motion computated by
`PyEphem <http://rhodesmill.org/pyephem/>`__. The accuracy of solar
terms may be within one mitute.

Requirements
------------

-  Python 2.x / 3.x
-  `PyEphem <http://rhodesmill.org/pyephem/>`__
-  `pytz <http://pytz.sourceforge.net/>`__
-  `jdcal <https://pypi.python.org/pypi/jdcal>`__

Installation
------------

.. code:: bash

    pip install pyephem
    pip install pytz
    pip install jdcal
    python setup.py install

Example & Usage
---------------

Calculating solar terms in a year
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

The solar terms of 2015 for English (in UTC).

.. code:: py

    >>> import eacal
    >>> from datetime import datetime
    >>> c = eacal.EACal()
    >>> for x in c.get_annual_solar_terms(2015):
    ...     print "%-25s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    ... 
    minor cold                2015-01-05 16:20 UTC
    major cold                2015-01-20 09:43 UTC
    start of spring           2015-02-04 03:58 UTC
    rain water                2015-02-18 23:49 UTC
    awakening of insects      2015-03-05 21:55 UTC
    vernal equinox            2015-03-20 22:45 UTC
    ...
    major snow                2015-12-07 10:53 UTC
    winter solstice           2015-12-22 04:47 UTC

The solar terms of 2015, for Traditional Chinese (in Hong Kong Time),
Simplified Chinese (in Chinese Standard Time), Japanese (in Japan
Standard Time), Korean (in Korea Standard Time), and Vietnamese (in
Indochina Time).

.. code:: py

    >>> import eacal
    >>> from datetime import datetime
    >>> c_t = eacal.EACal(zh_t=True)
    >>> for x in c_t.get_annual_solar_terms(2015):
    ...     print "%s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    小寒 2015-01-06 00:20 HKT
    大寒 2015-01-20 17:43 HKT
    立春 2015-02-04 11:58 HKT
    雨水 2015-02-19 07:49 HKT
    驚蟄 2015-03-06 05:55 HKT
    春分 2015-03-21 06:45 HKT
    ...
    大雪 2015-12-07 18:53 HKT
    冬至 2015-12-22 12:47 HKT

    >>> c_s = eacal.EACal(zh_s=True)
    >>> for x in c_s.get_annual_solar_terms(2015):
    ...     print "%s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    小寒 2015-01-06 00:20 CST
    大寒 2015-01-20 17:43 CST
    立春 2015-02-04 11:58 CST
    雨水 2015-02-19 07:49 CST
    惊蛰 2015-03-06 05:55 CST
    春分 2015-03-21 06:45 CST
    ...
    大雪 2015-12-07 18:53 CST
    冬至 2015-12-22 12:47 CST

    >>> c_j = eacal.EACal(ja=True)
    >>> for x in c_j.get_annual_solar_terms(2015):
    ...     print "%s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    小寒 2015-01-06 01:20 JST
    大寒 2015-01-20 18:43 JST
    立春 2015-02-04 12:58 JST
    雨水 2015-02-19 08:49 JST
    啓蟄 2015-03-06 06:55 JST
    春分 2015-03-21 07:45 JST
    ...
    大雪 2015-12-07 19:53 JST
    冬至 2015-12-22 13:47 JST

    >>> c_k = eacal.EACal(ko=True)
    >>> for x in c_k.get_annual_solar_terms(2015):
    ...     print "%s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    소한 2015-01-06 01:20 KST
    대한 2015-01-20 18:43 KST
    입춘 2015-02-04 12:58 KST
    우수 2015-02-19 08:49 KST
    경칩 2015-03-06 06:55 KST
    춘분 2015-03-21 07:45 KST
    ...
    대설 2015-12-07 19:53 KST
    동지 2015-12-22 13:47 KST

    >>> c_v = eacal.EACal(vi=True)
    >>> for x in c_v.get_annual_solar_terms(2015):
    ...     print "%-12s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    Tiểu hàn     2015-01-05 23:20 ICT
    Đại hàn      2015-01-20 16:43 ICT
    Lập xuân     2015-02-04 10:58 ICT
    Vũ thủy      2015-02-19 06:49 ICT
    Kinh trập    2015-03-06 04:55 ICT
    Xuân phân    2015-03-21 05:45 ICT
    ...
    Đại tuyết    2015-12-07 17:53 ICT
    Đông chí     2015-12-22 11:47 ICT

The solar terms of 2015 in North American Eastern Time Zone.

.. code:: py

    >>> import eacal
    >>> import pytz
    >>> from datetime import datetime
    >>> c = eacal.EACal(tz=pytz.timezone('America/New_York'))
    >>> for x in c.get_annual_solar_terms(2015):
    ...     print "%-25s %s" % (x[0], datetime.strftime(x[2], "%Y-%m-%d %H:%M %Z"))
    minor cold                2015-01-05 11:20 EST
    major cold                2015-01-20 04:43 EST
    start of spring           2015-02-03 22:58 EST
    rain water                2015-02-18 18:49 EST
    awakening of insects      2015-03-05 16:55 EST
    vernal equinox            2015-03-20 18:45 EDT  # in DST
    ...
    major snow                2015-12-07 05:53 EST
    winter solstice           2015-12-21 23:47 EST

Calculating sexagenary cycles
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

Calculating the cyclic year of 2015.

.. code:: py

    >>> import eacal
    >>> print(eacal.EACal().get_cycle_year(2015))
    wood-yin goat   # 乙未
    >>> print(eacal.EACal(zh_t=True).get_cycle_year(2015))
    乙未
    >>> print(eacal.EACal(vi=True).get_cycle_year(2015))
    ất mùi   # 乙未

Calculating the Cyclic month of May 2015.

.. code:: py

    >>> import eacal
    >>> print(eacal.EACal().get_cycle_month(2015, 5))
    metal-yin snake   # 辛巳
    >>> print(eacal.EACal(ja=True).get_cycle_month(2015, 5))
    辛巳
    >>> print(eacal.EACal(ko=True).get_cycle_month(2015, 5))
    신사   # 辛巳

Calculating the Cyclic day of 10th, May 2015.

.. code:: py

    >>> import eacal
    >>> from datetime import date
    >>> print(eacal.EACal().get_cycle_day(date(2015, 5, 10)))
    fire-yang dog   # 丙戌
    >>> print(eacal.EACal(zh_s=True).get_cycle_day(date(2015, 5, 10)))
    丙戌
    >>> print(eacal.EACal(vi=True).get_cycle_day(date(2015, 5, 10)))
    bính tuất   # 丙戌

Calculating the Cyclic year, month, and day around the start of spring
in 2015.

.. code:: py

    >>> import eacal
    >>> from datetime import datetime
    >>> c = eacal.EACal()     # for English, in UTC
    >>> print('|'.join(c.get_cycle_ymd(datetime(2015, 2, 3))))
    wood-yang horse|fire-yin ox|metal-yang dog     # 甲午年 丁丑月 庚戌日
    >>> print('|'.join(c.get_cycle_ymd(datetime(2015, 2, 4))))
    wood-yin goat|earth-yang tiger|metal-yin pig   # 乙未年 戊寅月 辛亥日 (cyclic year and cyclic month incremented at the start of spring)
    >>> print('|'.join(c.get_cycle_ymd(datetime(2015, 2, 5))))
    wood-yin goat|earth-yang tiger|water-yang rax  # 乙未年 戊寅月 壬子日
    >>> print(c.get_cycle_ymd(datetime(2015, 2, 3), id=True))
    (30, 13, 46)    # 30=wood-yang horse, 13=fire-yin ox, 46=metal-yang dog
    >>> print(c.get_cycle_ymd(datetime(2015, 2, 4), id=True))
    (31, 14, 47)    # 31=wood-yin goat, 14=earth-yang tiger, 47=metal-yin pig
    >>> print(c.get_cycle_ymd(datetime(2015, 2, 5), id=True))
    (31, 14, 48)    # 48=water-yang rax

Calculating Zassetsu
~~~~~~~~~~~~~~~~~~~~

.. code:: py

    >>> import eacal
    >>> import pytz
    >>> from datetime import datetime, timedelta
    >>> c = eacal.EACal(tz=pytz.timezone('Asia/Tokyo'))   # for English, in Japan Standard Time
    >>> for x in c.get_annual_jp_seasonal_days(2015):
    ...    if len(x) == 4:
    ...        print("%3d %s %s %s" % (x[1], datetime.strftime(x[2], "%Y-%m-%d"), datetime.strftime(x[3]-timedelta(days=1), "%Y-%m-%d"), x[0]))
    ...    else:
    ...        print("%3d %s %s" % (x[1], datetime.strftime(x[2], "%Y-%m-%d"), x[0]))
      1 2015-01-17 2015-02-03 doyō:winter
      2 2015-04-17 2015-05-05 doyō:spring
      3 2015-07-20 2015-08-07 doyō:summer
      4 2015-10-21 2015-11-07 doyō:autumn
     11 2015-03-18 2015-03-24 higan:spring
     12 2015-09-20 2015-09-26 higan:autumn
    101 2015-02-03 setsubun:the day before the start of spring
    102 2015-05-02 hachijū-hachi-ya:the 88th night after the start of spring
    103 2015-09-01 nihyaku-tōka:the 210th day after the start of spring
    111 2015-06-11 nyūbai:the beginning of rainy season
    112 2015-07-02 hangeshō:the end of field work

    >>> c_j = eacal.EACal(ja=True)   # for Japanese, in Japan Standard Time
    >>> for x in c_j.get_annual_jp_seasonal_days(2015):
    ...    if len(x) == 4:
    ...        print("%3d %s %s %s" % (x[1], datetime.strftime(x[2], "%Y-%m-%d"), datetime.strftime(x[3]-timedelta(days=1), "%Y-%m-%d"), x[0]))
    ...    else:
    ...        print("%3d %s %s" % (x[1], datetime.strftime(x[2], "%Y-%m-%d"), x[0]))
      1 2015-01-17 2015-02-03 土用:冬
      2 2015-04-17 2015-05-05 土用:春
      3 2015-07-20 2015-08-07 土用:夏
      4 2015-10-21 2015-11-07 土用:秋
     11 2015-03-18 2015-03-24 彼岸:春
     12 2015-09-20 2015-09-26 彼岸:秋
    101 2015-02-03 節分
    102 2015-05-02 八十八夜
    103 2015-09-01 二百十日
    111 2015-06-11 入梅
    112 2015-07-02 半夏生

TODO
----

-  a method for finding days which have a specified sexagenary cycle.
-  regnal years
'''

setup(name = 'eacal',
      version = '0.0.1',
      author = 'Yoji Kiyota',
      author_email = 'kiyotayoji@gmail.com',
      description = 'Python modules for accessing East Asia calendars (Sexagenary cycle, Solar term, etc.)',
      long_description = long_description,
      license = 'LGPL',
      url = 'https://github.com/kiyota-yoji/EastAsiaCalendars',
      packages = ['eacal'],
      classifiers = [
          'Development Status :: 3 - Alpha',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
          'Natural Language :: Chinese (Simplified)',
          'Natural Language :: Chinese (Traditional)',
          'Natural Language :: English',
          'Natural Language :: Japanese',
          'Natural Language :: Korean',
          'Natural Language :: Vietnamese',
          'Programming Language :: Python :: 2',
          'Programming Language :: Python :: 2.6',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Topic :: Scientific/Engineering :: Astronomy',
      ]
)
